import asyncio
import json
import pathlib
import time
import typing

import click

from travelogues_extraction.controller.main import FromAlmaOutputToExcel


@click.command()
@click.option('--input-file', help='Use .xlsx file as input', type=click.Path(exists=True))
@click.option('--ac-column', type=click.STRING, help=
'The column of the input file, where the ac numbers are. The column name is the string in the first row of the column',
              )
@click.option('--start', default=0, type=click.INT, help='The first record of the input file to extract the data')
@click.option('--stop', default=None, type=click.INT, help='The last record of the input file to extract the data')
@click.option('--output-file', help='Generate this .xlsx file', type=click.Path())
@click.option('--log-file', default=None, help='If given, write a csv log that that file', type=click.Path())
@click.option('--deep-log-file', default=None, help='If given, write a deep json log that that file', type=click.Path())
def extract_data(input_file: str,
                 output_file: str,
                 ac_column: str,
                 start: typing.Optional[int] = 0,
                 stop: typing.Optional[int] = None,
                 log_file: typing.Optional[str] = None,
                 deep_log_file: typing.Optional[str] = None
                 ):
    """
    Takes an excel file with a column of ac numbers, gets data from alma, brings the data in excel like shape, and generates an excel file with this data
    """

    loop = asyncio.new_event_loop()
    loop.run_until_complete(_extract_data(input_file, output_file, ac_column, start, stop, log_file, deep_log_file))


async def _extract_data(input_file: str,
                        output_file: str,
                        ac_column: str,
                        start: typing.Optional[int] = 0,
                        stop: typing.Optional[int] = None,
                        log_file: typing.Optional[str] = None,
                        deep_log_file: typing.Optional[str] = None
                        ) -> typing.NoReturn:

    click.echo('Starting script at {}'.format(time.ctime()))

    input_file = pathlib.Path(input_file)
    output_path = pathlib.Path(output_file)

    converter = FromAlmaOutputToExcel(input_file, output_path.absolute(), slice(start, stop, 1), ac_column)
    await converter.runasync()
    await converter.close()
    converter.write()
    click.echo('Wrote data to {} at {}'.format(output_path, time.ctime()))
    if log_file:
        converter.log.to_csv(log_file)
        click.echo('Wrote log to {} at {}'.format(log_file, time.ctime()))
    else:
        click.echo('No log to {} at {}'.format(log_file, time.ctime()))
    if deep_log_file:
        with open(deep_log_file, 'w') as file:
            json.dump(converter.sub_log, file)
        click.echo('Wrote deep-log to {} at {}'.format(deep_log_file, time.ctime()))
    else:
        click.echo('No deep-log to {} at {}'.format(log_file, time.ctime()))

    click.echo('Finished at {}'.format(time.ctime()))

if __name__ == '__main__':
    extract_data()
